#include <stdio.h>
#include <avr\io.h>
#include <avr\interrupt.h>
#include "lcd.h"
#include <math.h>

// Citire valoare de pe un anumit canal ADC
int ADC_get(uint8_t channel){	

	// setare canal de pe care se face citirea
	ADMUX &= 0xE0;
	ADMUX |= channel;
	ADCSRA |= _BV(ADEN)|_BV(ADSC);
	
	// asteptare sa se termine o conversie
	while(!(ADCSRA & (1<<ADIF)));
	
	// returnare rezultat conversie
	return ADC;
	
}

// Initializare canal ADC
void ADC_init(int port){

	DDRA = ~_BV(port);
	ADMUX = _BV(REFS1) | _BV(REFS0) | port;
}

// Afisare tensiune
void afiseaza_tensiune(int tensiune){
	
	unsigned int intreg, virgula;
	char mesaj[20];
	float rez;
	
	// determinare tensiune
	rez  = ((float)tensiune * 2.5 ) / 1024 * 8;
	// determinare parte intreaga
	intreg = (int) trunc(rez);
	// determinare parte fractionara
	virgula = (int)trunc((rez - intreg) * 1000);
	
	if (intreg != 0)
		sprintf(mesaj, "%u,%02u V", intreg, virgula / 10);
	else
		sprintf(mesaj, "%u mV %d", virgula, tensiune);
	
	// afisare la LCD
	print_string(mesaj);
}

// Afisare curent
void afiseaza_curent(int tensiune, int stare){

	unsigned int intreg, virgula;
	char mesaj[20];
	float rez;
	
	// determinare tensiune
	rez  = ((float)tensiune * 2.5) / 1024;
	
	// determinare curent
	if (stare == 0)
		rez = rez / 9.4;
	else if (stare == 2)
		rez = rez / 0.25;
	
	// determinare parte intreaga
	intreg = (int) trunc(rez);
	// determinare parte fractionara
	virgula = (int)trunc((rez - intreg) * 1000);
	
	if (intreg != 0)
		sprintf(mesaj, "%u,%02u A %d", intreg, virgula / 10, tensiune);
	else
		sprintf(mesaj, "%u mA %d", virgula, tensiune);

	// afisare la LCD
	print_string(mesaj);

}

// Afisare rezistenta
void afiseaza_rezistenta(int tensiune, int stare){

	unsigned int  intreg, virgula;
	char mesaj[20];
	double rez;
	
	// determinare tensiune 
	rez = (double) tensiune;
	rez  = (rez * 2.5) / 1024;
	
	// determinare rezistenta 
	if (stare == 0)
		rez = (rez * 100) / (5 - rez);
	if (stare == 1)
		rez = (rez * 12800) / (5 - rez);
	if (stare == 2)
		rez = (rez * 1280) / (5 - rez) ;
	
	intreg = 0;
	intreg = (unsigned int) trunc(rez);
	virgula = (unsigned int)trunc((rez - intreg) * 1000);
	
	if (stare == 2){
		if (tensiune < 1023)
			sprintf(mesaj, "%u,%02u KOhm", intreg, virgula / 10);
		else sprintf(mesaj, "1");
	}
	else if (intreg > 1000){
		virgula = intreg % 1000;
		intreg = intreg / 1000;
		if (tensiune < 1023)
			sprintf(mesaj, "%u,%02u KOhm", intreg, virgula / 10);
		else sprintf(mesaj, "1");
	}
	else if(intreg < 1000){
		sprintf(mesaj, "%u,%02u Ohm", intreg,virgula / 10);
	}
	else sprintf(mesaj, "1");
	
	// afisare la LCD
	print_string(mesaj);

}

// Determinare stare a multimetrului
int determina_stare(void){

	char s[20];
	int st;
	
	s[0] =  !((PIND & (1 << PD4)));
	s[1] =  !((PIND & (1 << PD5)));
	s[2] = !((PIND & (1 << PD6)));
	s[3] =   !((PIND & (1 << PD7)));
	s[4] = '\0';
	
	if (s[1] == 0)
		PORTC &= ~_BV(5);
	else 
		PORTC |= _BV(5);
	if (s[2] == 1)
		PORTC |= _BV(6);
	else
		PORTC &= ~_BV(6);
	if (s[3] == 1)
		PORTC |= _BV(7);
	else
		PORTC &= ~_BV(7);
	
	st = s[1] * 4 + s[2] * 2 + s[3] * 1;
	
	// afisare stare la LCD
	switch (st){
	case 0:
		Gotoxy(10,0);
		print_string("Tensiune");
		Gotoxy(10,1);
		print_string("0 - 20V");
		break;	
	case 1:
		Gotoxy(10,0);
		print_string("Curent");
		Gotoxy(10,1);
		sprintf(s, "0 - 250mA");
		break;	
	case 2:
		Gotoxy(10,0);
		print_string("Curent");
		Gotoxy(10,1);
		sprintf(s, "250mA - 2.5 A");
		break;	
	case 3:
		Gotoxy(10,0);
		print_string("Curent");
		Gotoxy(10,1);
		sprintf(s, "2.5A - 10A");
		break;	
	case 4:
		Gotoxy(10,0);
		print_string("Rezistenta");
		Gotoxy(10,1);
		sprintf(s, "0 - 100 Ohmi");
		break;	
	case 5:
		Gotoxy(10,0);
		print_string("Rezistenta");
		Gotoxy(10,1);
		sprintf(s, "100 Ohmi - 10K");
		break;	
	case 6:
		Gotoxy(10,0);
		print_string("Rezistenta");
		Gotoxy(10,1);
		sprintf(s, "10K - 10M");
		break;	
	}
	
	print_string(s);
	
	return st;
	
}

int main(void)
  {
	
	int tensiune;
	int stare;
	
	MCUCSR = 0x80;
	
	// initializare porturi
	DDRC=0xFF;
	DDRD=0xFF;
	PORTD = 0xFF;
	
	// initializare LCD
	Lcd_Init();

	// intializare ADC
	ADC_init(PA0);
	ADC_init(PA1);
	ADC_init(PA2);
	ADC_init(PA3);
	ADC_init(PA4);
	ADC_init(PA5);
	ADC_init(PA6);
	
	
	while(1){
	
		// stergere ecran
		Lcd_Clear();

		// determinare stare
		stare = determina_stare();

		// citire tensiune de la ADC si afisare rezultat
		Gotoxy(10,3);
		switch(stare){
		case 0:
			ADC_init(PA0);
			tensiune = ADC_get(PA0);
			afiseaza_tensiune(tensiune);
			break;
		case 1:
			ADC_init(PA1);
			tensiune = ADC_get(PA1);
			afiseaza_curent(tensiune, 0);
			break;
		case 2:
			ADC_init(PA2);
			tensiune = ADC_get(PA2);
			afiseaza_curent(tensiune, 1);
			break;
		
		case 3:
			ADC_init(PA3);
			tensiune = ADC_get(PA3);
			afiseaza_curent(tensiune, 2);
			break;
		case 4:
			ADC_init(PA4);
			tensiune = ADC_get(PA4);
			afiseaza_rezistenta(tensiune, 0);
			break;
		case 5:
			ADC_init(PA5);
			tensiune = ADC_get(PA5);
			afiseaza_rezistenta(tensiune, 1);
			break;
		case 6:
			ADC_init(PA6);
			tensiune = ADC_get(PA6);
			afiseaza_rezistenta(tensiune, 2);
			break;
		}
		
		_delay_ms(500);
  }
 
}

